<?php


namespace WPFunnels\Modules\Frontend\Checkout;

use WPFunnels\Data_Store\Wpfnl_Steps_Store_Data;
use WPFunnels\Frontend\Module\Wpfnl_Frontend_Module;
use WPFunnels\Frontend\Wpfnl_Public;
use WPFunnels\Meta\Wpfnl_Default_Meta;
use WPFunnels\Wpfnl;
use WPFunnels\Wpfnl_functions;
use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;
use WPFunnels\Modules\Frontend\Checkout\Variable\Wpfnl_Variable_Product;
use WPFunnels\Modules\Frontend\Checkout\Single\Wpfnl_Single_Product;
use function cli\err;
use Wpfnl_Public_Type_Factory;

class Module extends Wpfnl_Frontend_Module
{
	public $funnel_id;

	public $step_id;

	// const CHECKOUT = "";

	public function __construct()
	{

		/* set checkout flag */
		add_filter('woocommerce_is_checkout', [$this, 'checkout_flag'], 9999);
		add_action('woocommerce_checkout_update_order_meta', [$this, 'save_checkout_fields'], 10, 2);

		/* initialize cart data */
		add_action('wp', [$this, 'initialize_cart_data']);

		add_action('wp', [$this, 'init_wc_actions']);
		/* register checkout shortcode */
		add_shortcode('wpfunnels_checkout', array($this, 'render_checkout_shortcode'));
		add_action('wp_ajax_wpfnl_next_button_ajax', [$this, 'wpfnl_next_button_ajax']);
		add_action('wp_ajax_nopriv_wpfnl_next_button_ajax', [$this, 'wpfnl_next_button_ajax']);

		add_action( 'wp_ajax_wpfnl_order_bump_ajax', [$this, 'wpfnl_order_bump_ajax']);
		add_action( 'wp_ajax_nopriv_wpfnl_order_bump_ajax', [$this, 'wpfnl_order_bump_ajax']);

		add_action( 'wp_ajax_wpfnl_get_course_details', [$this, 'wpfnl_get_course_details']);
		add_action( 'wp_ajax_nopriv_wpfnl_get_course_details', [$this, 'wpfnl_get_course_details']);

		add_action( 'wp_ajax_wpfnl_update_variation', [$this, 'wpfnl_update_variation']);
		add_action( 'wp_ajax_nopriv_wpfnl_update_variation', [$this, 'wpfnl_update_variation']);

		add_action( 'woocommerce_before_calculate_totals', [$this, 'custom_price_to_cart_item'], 9999);

		add_action( 'wp_ajax_nopriv_wpfnl_checkout_cart', [$this, 'wpfnl_checkout_cart']);
		add_filter('woocommerce_cart_product_out_of_stock_message', [$this,'wpfnl_out_of_stock_message_in_checkout'], 10, 2);

		add_action( 'wpfunnels/before_checkout_form', array($this, 'before_checkout_form_actions') );

		add_action( 'woocommerce_after_order_notes', array( $this, 'wpfnl_set_checkout_id' ), 999 );
		add_action( 'woocommerce_login_form_end', array( $this, 'wpfnl_set_checkout_id' ), 999 );
		add_filter( 'woocommerce_login_redirect', [$this,'wpfnl_redirect_after_login'], 10, 2 );
		add_action( 'woocommerce_cart_calculate_fees', [$this,'wpfnl_checkout_discount'], 10);
		add_action('woocommerce_cart_totals_get_fees_from_cart_taxes', [$this, 'exclude_cart_fees_taxes'], 10 ,3);
		add_action( 'wpfunnels/after_optin_submit', array( $this, 'get_optin_data_checkout' ), 10, 4 );
		add_filter( 'woocommerce_checkout_fields', array($this, 'set_option_data_in_checkout_filed'),10 );
		// add_filter( 'woocommerce_subscriptions_display_recurring_fee_totals', array($this, 'remove_discount_from_recurring_totals'),10, 2 );

		// add_filter( 'woocommerce_subscriptions_display_recurring_subscription_totals', array($this, 'remove_discount_from_recurring_totals'),10, 2 );

        
	}


	/**
	 * set checkout flag to true if this is checkout step type
	 *
	 * @param $is_checkout
	 * @return bool
	 *
	 * @since 1.0.0
	 */
	public function checkout_flag($is_checkout)
	{
		if (Wpfnl_functions::check_if_this_is_step_type('checkout')) {
			$is_checkout = true;
		}
		return $is_checkout;
	}


	/**
	 * init WooCommerce related actions
	 */
	public function init_wc_actions() {
		add_action( 'woocommerce_after_order_notes', [$this, 'checkout_shortcode_metas']);
		$this->display_coupon_field();
		$this->enable_checkout_hook();
	}


	/**
	 * Enable checkout hook
	 */
	private function enable_checkout_hook(){

		if ( Wpfnl_functions::check_if_this_is_step_type('checkout') ) {

			if( is_plugin_active( 'woocommerce-german-market/WooCommerce-German-Market.php' )){
				add_action( 'woocommerce_checkout_order_review','woocommerce_checkout_payment' );
			}
		}
	}


	/**
	 * save checkout metas
	 *
	 * @param $order_id
	 * @param $posted
	 *
	 * @since 1.0.0
	 */
	public function save_checkout_fields( $order_id, $posted )
	{

		if (isset($_POST['_wpfunnels_checkout_id'])) {
			$checkout_id = sanitize_text_field($_POST['_wpfunnels_checkout_id']);
			$checkout_id = wc_clean($checkout_id);
			update_post_meta($order_id, '_wpfunnels_checkout_id', $checkout_id);
			if (isset($_POST['_wpfunnels_funnel_id'])) {
				$funnel_id = sanitize_text_field($_POST['_wpfunnels_funnel_id']);
				$funnel_id = wc_clean($funnel_id);
				update_post_meta($order_id, '_wpfunnels_funnel_id', $funnel_id);
			}
			if (isset($_POST['_wpfunnels_order_unique_identifier'])) {
				$unique_identifier = sanitize_text_field($_POST['_wpfunnels_order_unique_identifier']);
				$unique_identifier = wc_clean($unique_identifier);
				update_post_meta($order_id, '_wpfunnels_order_unique_identifier', $unique_identifier);
			}
			$orderBumpProducts = get_post_meta( $_POST['_wpfunnels_checkout_id'], 'order-bump-settings' , true );

			foreach( $orderBumpProducts as $key => $orderBumpProduct ){
				if ( isset( $_POST['_wpfunnels_order_bump_product_'.$key] ) ) {

					$ob_product_id = wc_clean( sanitize_text_field( wp_unslash( $_POST['_wpfunnels_order_bump_product_'.$key] ) ) );
					update_post_meta( $order_id, '_wpfunnels_order_bump_product', $ob_product_id );
					$order = wc_get_order( $order_id );
					if( $order && !is_wp_error( $order ) ){
						foreach ($order->get_items() as $order_item_id => $order_item) {

							if($_POST['_wpfunnels_order_bump_product_'.$key] == $order_item['product_id']){
								wc_add_order_item_meta($order_item_id, '_wpfunnels_order_bump', 'yes');
							}
						}
					}
				}
			}
		}

		if (isset($funnel_id)) {
			$order = wc_get_order($order_id);

			$funnel_id = Wpfnl_functions::get_funnel_id_from_step( $_POST['_wpfunnels_checkout_id'] );

			if( $funnel_id ){
				$discount = Wpfnl_functions::discount_on_checkout( $_POST['_wpfunnels_checkout_id'] );
				$main_products = get_post_meta($_POST['_wpfunnels_checkout_id'], '_wpfnl_checkout_products', true );
				$is_gbf = get_post_meta( $funnel_id, 'is_global_funnel', true);
				if( 'yes' === $is_gbf ){
                    if(isset( $_COOKIE['wpfunnels_global_funnel_product'] ) ){
						$main_products  = json_decode( wp_unslash( $_COOKIE['wpfunnels_global_funnel_product'] ), true );
					}
                }
				$total = Wpfnl_functions::calculate_main_product_total( $main_products , WC()->cart->cart_contents, $_POST['_wpfunnels_checkout_id'] );

				if( is_array($discount) ){
					if( $discount['discountOptions'] !== 'original' ){
						$discount_amount = Wpfnl_functions::calculate_discount_amount( $discount['discountOptions'], $discount['mutedDiscountValue'], $total );
						$discount_amount = filter_var($discount_amount, FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION);
						update_post_meta( $order_id, '_cart_discount', $discount_amount );
						$cart_total = Wpfnl_functions::calculate_cart_total( $main_products , WC()->cart->cart_contents, $_POST['_wpfunnels_checkout_id'] );
						if ( !wc_prices_include_tax() ) {
							$cart_total = $cart_total - $discount_amount;
							foreach ($order->get_items(array('tax')) as $item_id => $line_item) {
								$order_product_detail = $line_item->get_data();
								if( isset($order_product_detail['tax_total']) &&  isset($order_product_detail['rate_percent']) ){
									$prev_tax = $order_product_detail['tax_total'];
									$tax = ($cart_total*$order_product_detail['rate_percent'])/100;
									wc_update_order_item_meta($item_id, 'tax_total', $tax);
									wc_update_order_item_meta($item_id, 'tax_amount', $tax);

								}
							}
							$total = $cart_total + $tax;

						}else{
							$total = $cart_total - $discount_amount;
						}
						$shipping_total = $order->get_shipping_total();
						$shipping_tax   = $order->get_shipping_tax();
						$total = $total + $shipping_total + $shipping_tax;
						update_post_meta( $order_id, '_order_total', $total );
					}
				}
			}

    		$user_id = $order->get_user_id();
			$orders[sanitize_text_field($_POST['_wpfunnels_checkout_id'])] = $order_id;
			WC()->session->set('wpfnl_orders_'.$user_id.'_'.$funnel_id, $orders);
			do_action( 'wpfunnels/funnel_order_placed', $order_id, $funnel_id );
			WC()->cart->empty_cart();
		}
	}


	/**
	 * Automatically add product to checkout
	 *
	 * @param array $product_param
	 * @throws \Exception
	 *
	 * @since 1.0.0
	 */
	public function initialize_cart_data( $product_param = [] ) {
		if (is_admin()) {
			return;
		}
		if (isset($_GET['removed_item'])) {
			return;
		}

		global $post;

		if( !$post ){
			return false;
		}
		if ( wp_doing_ajax() || isset( $_GET['wc-ajax'] ) ) {
			$checkout_id = Wpfnl_functions::get_checkout_id_from_post($_POST);
		} else {
			$checkout_id = $post->ID;
		}

		$funnel_id = get_post_meta($checkout_id, '_funnel_id', true);
		if( !$funnel_id || 'lms' === get_post_meta( $funnel_id, '_wpfnl_funnel_type',true ) ){
			return false;
		}

		if ( Wpfnl_functions::check_if_this_is_step_type_by_id( $checkout_id, 'checkout' ) || Wpfnl_functions::check_if_this_is_step_type('checkout' ) ) {
			// Only for express checkout
			if( \WPFunnels\Wpfnl_functions::is_wpfnl_pro_activated() && Wpfnl_functions::maybe_express_checkout( $checkout_id ) ){
				remove_action( 'woocommerce_checkout_order_review', 'woocommerce_order_review' );
				add_filter( 'woocommerce_update_order_review_fragments', [$this,'wpfnl_express_checkout_shipping_table_update'], 10, 1);
			}

			if( !wp_doing_ajax() ){
				$funnel_id = get_post_meta($checkout_id, '_funnel_id', true);
				$is_global_funnel = get_post_meta( $funnel_id, 'is_global_funnel', true );

				$product_array = get_post_meta($checkout_id, '_wpfnl_checkout_products', true);
				$product_array = apply_filters('wpfunnels/funnel_products', $product_array, $funnel_id, $checkout_id);

				\WC()->cart->empty_cart();
				if (!is_array($product_array) || empty($product_array[0]['id'])) {
					$product_array = $product_param;
					wc_clear_notices();
					wc_add_notice( __('No product is added to the funnel. Please add product from checkout step settings.', 'wpfnl' ), 'error' );
				}

				$coupon = get_post_meta($checkout_id, '_wpfnl_checkout_discount', true);
				if (is_array($coupon) && !empty($coupon)) {
					$coupon = reset($coupon);
				}
				if ($product_array) {
					foreach ($product_array as $product) {

						if (isset($product["id"]) && isset($product["quantity"])) {

							$product_id = $product["id"];
							$quantity = $product["quantity"];
							$found = false;


							//check if product already in cart
							if (sizeof(WC()->cart->get_cart()) > 0) {
								foreach (WC()->cart->get_cart() as $cart_item_key => $values) {
									$_product = $values['data'];
									if ($_product->get_id() == $product_id) {
										$found = true;
									}
								}
								// if product not found, add it
								if (!$found) {

									if( isset($product['variation_id']) ){
										// Store product obj in $_product variable
										$_product = wc_get_product($product['variation_id']);
									}else{
										// Store product obj in $_product variable
										$_product = wc_get_product($product_id);
									}

									if( $_product ){
										//check if product type is variable or not from $_product object
										if ( !($_product->get_type() == 'variable') ) {

											// check if product has variation id or not from $product array
											if( isset($product['variation_id']) && isset($product['variations']) ){

												$ob_cart_item_data = [
													'custom_price' 	=> get_post_meta($product['variation_id'], '_price', true) ? get_post_meta($product['variation_id'], '_price', true) : get_post_meta($product['variation_id'], '_regular_price', true)
												];

												$product_cart_id = WC()->cart->generate_cart_id( $product_id );
												$cart_item_key = WC()->cart->find_product_in_cart( $product_cart_id );

												foreach (WC()->cart->get_cart() as $cart_item_key => $cart_item) {

													if ($cart_item['variation_id'] == $product['variation_id'] ) {
														//remove single product
														WC()->cart->remove_cart_item($cart_item_key);
													}
												}
												WC()->cart->add_to_cart( $product_id, $quantity, $product['variation_id'], $product['variations'], $ob_cart_item_data);

											}else{
												$__product = wc_get_product( $product_id );
												if( $__product ){
													$is_perfect_variation = false;
													if(  'variation' === $__product->get_type() || 'subscription_variation' == $__product->get_type() ){
														$is_perfect_variation = Wpfnl_functions::is_perfect_variations( $product_id );
														if( $is_perfect_variation['status'] ){
															$price_type = $this->get_product_price_type( $funnel_id, $checkout_id );
															if( $price_type == 'original' ||  $price_type == 'sale' ){
																WC()->cart->add_to_cart( $product_id, $quantity );
															}else{
																$custom_price['custom_price'] = $__product->get_regular_price() ? $__product->get_regular_price() : $__product->get_price();
																WC()->cart->add_to_cart( $product_id, $quantity, '' , '' , $custom_price );
															}

															WC()->cart->add_to_cart($product_id, $quantity);
														}else{
															$this->add_variaiton_product_in_cart( $funnel_id, $checkout_id, $product_id, $quantity );
														}
													}else{
														$price_type = $this->get_product_price_type( $funnel_id, $checkout_id );
														if( $price_type == 'original' ||  $price_type == 'sale' ){
															WC()->cart->add_to_cart( $product_id, $quantity );
														}else{
															$custom_price['custom_price'] = $__product->get_regular_price() ? $__product->get_regular_price() : $__product->get_price();
															WC()->cart->add_to_cart( $product_id, $quantity, '' , '' , $custom_price );
														}
													}
												}
											}

										}else {
											$variations = $_product->get_available_variations();

											$this->add_default_variation($funnel_id, $checkout_id, $product_id, $_product, $variations, $quantity);
										}
									}

								}
							} else {
								$_product = isset($product['variation_id']) && $product['variation_id'] ? wc_get_product($product['variation_id']) :  wc_get_product($product_id);
								if( $_product ){
									//check if product type is variable or not from $_product object
									if ( 'variable' === $_product->get_type() || 'variable-subscription' == $_product->get_type() ) {
                                        $variations = $_product->get_available_variations();
										$this->add_default_variation( $funnel_id, $checkout_id, $product_id, $_product, $variations, $quantity);

									}else {
										// check if product has variation id or not from $product array
										if( isset($product['variation_id']) && isset($product['variations']) ){
											$price_type = $this->get_product_price_type( $funnel_id, $checkout_id );
											if( $price_type == 'original' ||  $price_type == 'sale' ){
												$ob_cart_item_data = [
													'custom_price' 	=> get_post_meta($product['variation_id'], '_price', true) ? get_post_meta($product['variation_id'], '_price', true) : get_post_meta($product['variation_id'], '_regular_price', true)
												];
											}else{
												$ob_cart_item_data = [
													'custom_price' 	=> get_post_meta($product['variation_id'], '_regular_price', true) ? get_post_meta($product['variation_id'], '_regular_price', true) : get_post_meta($product['variation_id'], '_price', true)
												];

											}

											WC()->cart->add_to_cart( $product_id, $quantity, $product['variation_id'], $product['variations'], $ob_cart_item_data);
										}else{
											$__product = wc_get_product( $product_id );
											if( $__product ){
												$is_perfect_variation = false;
												if( $__product->get_type() == 'variation' ){
													$is_perfect_variation = Wpfnl_functions::is_perfect_variations( $product_id );
													if( $is_perfect_variation['status'] ){

														$price_type = $this->get_product_price_type( $funnel_id, $checkout_id );

														if( $price_type == 'original' ||  $price_type == 'sale' ){
															WC()->cart->add_to_cart( $product_id, $quantity );
														}else{

															$custom_price['custom_price'] = $__product->get_regular_price() ? $__product->get_regular_price() : $__product->get_price();
															WC()->cart->add_to_cart( $product_id, $quantity, '' , '' , $custom_price );
														}
													}else{
														$this->add_variaiton_product_in_cart( $funnel_id,$checkout_id, $product_id, $quantity );
													}
												}else{
													$price_type = $this->get_product_price_type( $funnel_id, $checkout_id );

													if( $price_type == 'original' ||  $price_type == 'sale' ){
														WC()->cart->add_to_cart( $product_id, $quantity );
													}else{
														$custom_price['custom_price'] = $__product->get_regular_price() ? $__product->get_regular_price() : $__product->get_price();
														WC()->cart->add_to_cart( $product_id, $quantity, '' , '' , $custom_price );
													}
												}
											}
										}
									}
								}

							}
						}
					}
				}

				WC()->session->set('order_bump_accepted', 'no');

				if (!empty($coupon)) {
					WC()->cart->add_discount($coupon);
				}

				if( $is_global_funnel && 'yes' === $is_global_funnel ) {
					$gbf_coupons = [];
					if(isset( $_COOKIE['wpfnl_gbf_coupon'] ) ){
						$gbf_coupons  = json_decode( wp_unslash( $_COOKIE['wpfnl_gbf_coupon'] ), true );
					}

					if (!empty($gbf_coupons)) {
						foreach( $gbf_coupons as $cp ){
							if(!in_array($cp, WC()->cart->applied_coupons )){
								WC()->cart->add_discount($cp);
							}
						}
					}
				}
			}

		}
	}


	private function add_variaiton_product_in_cart($funnel_id, $checkout_id, $product_id, $quantity){

			$product = wc_get_product( $product_id );
			if( $product ){
				$selected_attr = array();
				$attributes = $product->get_attributes();
				foreach($attributes as $attribute_key=>$attribute_value){

					if( $attribute_value ){
						$selected_attr[] = $attribute_key;
						$default_attr['attribute_'.$attribute_key] = $attribute_value;
					}
				}

				if( $product->get_parent_id() ){
					$_product = wc_get_product( $product->get_parent_id() );
					if( $_product ){
						$attributes = $_product->get_attributes();
						foreach ( $attributes as $key => $value) {
							if( !in_array($key,$selected_attr) ){
								$attr_value = $_product->get_attribute( $key );
								$attr_value = strtolower($attr_value);
								if (strpos($attr_value, '|')) {
									$attr_array = explode("|",$attr_value);
								}else{
									$attr_array = explode(",",$attr_value);
								}
								$default_attr['attribute_'.$key] = $attr_array[0];

							}
						}
					}

					$price_type = $this->get_product_price_type( $funnel_id, $checkout_id );
					if( $price_type == 'original' ||  $price_type == 'sale' ){
						$cart_item_data = [
							'custom_price' 	=> get_post_meta($product_id, '_price', true) ? get_post_meta($product_id, '_price', true) : get_post_meta($product_id, '_regular_price', true)
						];
					}else{
						$cart_item_data = [
							'custom_price' 	=> get_post_meta($product_id, '_regular_price', true) ? get_post_meta($product_id, '_regular_price', true) : get_post_meta($product_id, '_price', true)
						];
					}
					WC()->cart->add_to_cart( $product->get_parent_id(), $quantity, $product_id, $default_attr, $cart_item_data);
				}
			}
	}


	/**
	 * Update shiiping markup for express checkout
	 *
	 * @since 2.4.18
	 */
	public function wpfnl_express_checkout_shipping_table_update( $fragments ) {
		ob_start();
		require WPFNL_DIR.'/woocommerce/templates/checkout/review-order.php';
		$woocommerce_shipping_methods = ob_get_clean();
		$fragments['.woocommerce-checkout-review-order-table'] = $woocommerce_shipping_methods;
		return $fragments;
	}



	/**
	 * Add default variation to cart
	 *
	 * @param $product
	 * @param $variations
	 * @param $quantity
	 * @throws \Exception
	 */
	private function add_default_variation($funnel_id, $checkout_id,$product_id,$product,$variations,$quantity){
		$i = 0;
		$formatted_variation = [];
		$is_default_variation = false;
		foreach ($variations as $variation) {

			if($variation['is_in_stock'] ){

				if($product->get_default_attributes()){
					$attributes = $product->get_attributes();
					$def_attributes = $product->get_default_attributes();
					foreach($attributes as $attribute_key=>$attribute_value){

						$def_attr = $product->get_default_attributes();
						if(isset($def_attr[$attribute_key])){
							$attribute_name = str_replace( 'attribute_', '', $attribute_key );
							$default_value = $product->get_variation_default_attribute($attribute_name);
							$formatted_variation['attribute_'.$attribute_name] = $default_value;
							$is_default_variation = true;
							// $default_attr[] = $default_value;

						}
						else{
							$is_default_variation = true;
							$attribute_name = str_replace( 'attribute_', '', $attribute_key );
							$attr_value = $product->get_attribute( $attribute_name );
							$attr_value = strtolower($attr_value);
							if (strpos($attr_value, '|')) {
								$attr_array = explode("|",$attr_value);
							}else{
								$attr_array = explode(",",$attr_value);
							}
							$formatted_variation['attribute_'.$attribute_name] = $attr_array[0];
							// $default_attr[] = $attr_array[0];
						}

					}
				}else{
					if($i==0){
						$attributes = $product->get_attributes();
						foreach ( $attributes as $key => $value) {
							$attr_value = $product->get_attribute( $key );
							$attr_value = strtolower($attr_value);
							if (strpos($attr_value, '|')) {
								$attr_array = explode("|",$attr_value);
							}else{
								$attr_array = explode(",",$attr_value);
							}
							$is_custom_default_variation = true;
							$formatted_variation['attribute_'.$key] = $attr_array[0];
						}
					}


					// WC()->cart->add_to_cart($variation['variation_id'], $quantity);
				}

				if( $is_default_variation || $is_custom_default_variation){
					$variation_id = (new \WC_Product_Data_Store_CPT())->find_matching_product_variation(
						new \WC_Product($product_id),
						$formatted_variation
					);

					if( !$variation_id ){
						$_product = wc_get_product($product_id);
						if( $_product ){
							$variations = $_product->get_available_variations();
							$variations_id = wp_list_pluck( $variations, 'variation_id' );
							$variation_id = $variations_id[0];
							$formatted_variation = wc_get_product_variation_attributes($variation_id);
						}
					}

					$price_type = $this->get_product_price_type( $funnel_id, $checkout_id );

					if( $price_type == 'original' ||  $price_type == 'sale' ){
						$cart_item_data = [
							'custom_price' 	=> get_post_meta($variation_id, '_price', true) ? get_post_meta($variation_id, '_price', true) : get_post_meta($variation_id, '_regular_price', true)
						];
					}else{
						$cart_item_data = [
							'custom_price' 	=> get_post_meta($variation_id, '_regular_price', true) ? get_post_meta($variation_id, '_regular_price', true) : get_post_meta($variation_id, '_price', true)
						];
					}

					WC()->cart->add_to_cart( $product_id, $quantity, $variation_id, $formatted_variation, $cart_item_data);
					break; // Stop the main loop
				}
				$i++;
			}
		}
	}


	/**
	 * render content for wpfunnels_checkout shortcode
	 *
	 * @param $atts
	 * @return string
	 *
	 * @since 2.0.3
	 */
	public function render_checkout_shortcode($atts)
	{
		$atts = shortcode_atts(
			array(
				'id' => 0,
			),
			$atts
		);
		$checkout_id = intval($atts['id']);
		if (is_admin()) {
			if (0 === $checkout_id && isset($_POST['id'])) {
				$checkout_id = intval($_POST['id']);
			}
		}

		if (empty($checkout_id)) {
			global $post;
			if(is_object($post)) {
				$checkout_id = intval($post->ID);
			}
		}
		if(!$checkout_id) {
			return '';
		}
		$checkout_layout = Wpfnl::get_instance()->meta->get_checkout_meta_value($checkout_id, 'wpfnl_checkout_layout', 'wpfnl-col-2');

		$output = '';
		ob_start();

		do_action('wpfunnels/before_gb_checkout_form', $checkout_id);

		$template_file = WPFNL_DIR . "public/modules/checkout/templates/checkout-template.php";
		include $template_file;

		do_action('wpfunnels/after_checkout_form', $checkout_id);

		$output .= ob_get_clean();
		return $output;

	}


	public function custom_price_to_cart_item($cart_object)
	{
		if (wp_doing_ajax() && !WC()->session->__isset('reload_checkout')) {
			foreach ($cart_object->cart_contents as $key => $value) {
				if (isset($value['custom_price'])) {
					$custom_price = floatval($value['custom_price']);
					$value['data']->set_price($custom_price);
				}
			}
		}
	}

	public function render_order_bump_popup()
	{
		$step_id = get_the_ID();
		$order_bump = get_post_meta($step_id, 'order-bump', true);
		$order_bump_settings = get_post_meta($step_id, 'order-bump-settings', true);
		require_once WPFNL_DIR . 'public/modules/checkout/templates-style/order-bump-template-style3.php';
	}



	/**
	 * insert helper hidden ids
	 *
	 * @since 1.0.0
	 */
	public function checkout_shortcode_metas()
	{
		if (Wpfnl_functions::check_if_this_is_step_type('checkout')) {
			global $post;
			$this->step_id = $post->ID;
			$step = new Wpfnl_Steps_Store_Data();
			$step->read($this->step_id);
			$this->funnel_id = $step->get_funnel_id();
			echo '<input type="hidden" class="input-hidden _wpfunnels_funnel_id" name="_wpfunnels_funnel_id" value="' . intval($this->funnel_id) . '">';
			echo '<input type="hidden" class="input-hidden _wpfunnels_checkout_id" name="_wpfunnels_checkout_id" value="' . intval($post->ID) . '">';
			echo '<input type="hidden" class="input-hidden _wpfunnels_order_unique_identifier" name="_wpfunnels_order_unique_identifier" value="' . uniqid() . '">';
		}
	}


	/**
	 * Next step button ajax
	 *
	 * @since 1.0.0
	 */
	public function wpfnl_next_button_ajax()
	{
		$step_id 	= sanitize_text_field($_POST['step_id']);
		$funnel_id 	= get_post_meta($step_id, '_funnel_id', true);
		$next_step 	= Wpfnl_functions::get_next_step($funnel_id, $step_id);
		$next_step       = apply_filters( 'wpfunnels/next_step_data', $next_step );
		if ( $next_step ) {

			$custom_url = Wpfnl_functions::custom_url_for_thankyou_page( $next_step['step_id'] );
            if( $custom_url ){
                $response =  $custom_url;
            }else{
				$next_step_id = $next_step['step_id'];
				$redirect_url = get_post_permalink($next_step_id);
				if ($redirect_url) {
					$response = $redirect_url;
				} else {
					$response = 'error';
				}
			}

			do_action( 'wpfunnels/trigger_cta', $step_id, $funnel_id );
			wp_send_json($response);
		}
		return false;
	}

	public function add_product_to_checkout()
	{
	}

	public function load_data()
	{
	}



	/**
	 * Calculate Discount.
	 * @since 1.1.5
	 */
	public function calculate_custom_price($discount_type, $discount_value, $product_price)
	{
		$custom_price = $product_price;

		if (!empty($discount_type)) {
			if ('discount-percentage' === $discount_type) {
				if ( $discount_value > 0 && $discount_value <= 100) {
					$custom_price = $product_price - (($product_price * $discount_value) / 100);
				}else{
					$custom_price = $product_price;
				}
			} elseif ('discount-price' === $discount_type) {
				if ($discount_value > 0 && $product_price >= $discount_value ) {
					$custom_price = $product_price - $discount_value;
				}else{
					$custom_price = $product_price;
				}
			}
		}

		return number_format($custom_price, 2);
	}

	/**
	 * Apply Coupon to product.
	 * @since 1.1.5
	 */
	public function apply_discount_coupon($discount_type, $discount_coupon)
	{
		$coupon_applied = false;

		if ('coupon' === $discount_type && !empty($discount_coupon)) {
			$discount_coupon = strtolower($discount_coupon);
			WC()->cart->add_discount($discount_coupon);
			$coupon_applied = true;
		}

		return $coupon_applied;
	}

	public function wpfnl_update_variation(){

		$variations 	= $_POST['variations'];
		$product_id 	= $_POST['variations'][0]['product_id'];
		$variation_id 	= $_POST['variations'][0]['variation_id'];
		$quantity 		= $_POST['variations'][0]['quantity'];

		$_product = wc_get_product($variation_id);
		if( $_product ){
			$attribute = $_product->get_attributes();
			foreach ( $attribute as $key => $value) {
				if( $value ){
					$formatted_variation['attribute_'.$key] = $value;
				}else{
					foreach( $variations as $variation ){
						if( $variation['attr'] == $key ){
							$formatted_variation['attribute_'.$key] = $variation['value'];
						}
					}
				}
			}
		}


		$variation_id = (new \WC_Product_Data_Store_CPT())->find_matching_product_variation(
			new \WC_Product($product_id),
			$formatted_variation
		);

		foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ) {
			if ($cart_item['product_id'] == $product_id) {
				WC()->cart->remove_cart_item($cart_item_key);
			}
		}
		$cart_item_data = [
			'custom_price' 	=> get_post_meta($variation_id, '_price', true) ? get_post_meta($variation_id, '_price', true) : get_post_meta($variation_id, '_regular_price', true)
		];

		WC()->cart->add_to_cart( $product_id, $quantity, $variation_id, $formatted_variation, $cart_item_data);
	}





	/**
	 * Order Bump Ajax Initialized
	 * When you click on add order bump button it will add the product to cart and click again to recover cart
	 *
	 * @since 1.1.
	 */
	public function wpfnl_get_course_details()
	{
		$course_id 		= filter_input(INPUT_POST, 'course_id', FILTER_VALIDATE_INT);
		$course_meta = Wpfnl_lms_learndash_functions::get_course_details_by_id($course_id);
		// ld_update_course_access( 1, $course_id, false );
		$data = [
			'status'  => 'success',
			'course'  => $course_meta,
		];

		wp_send_json( $data );

	}


	/**
	 * Order Bump Ajax Initialized
	 * When you click on add order bump button it will add the product to cart and click again to recover cart
	 *
	 * @since 1.1.
	 */
	public function wpfnl_order_bump_ajax() {
		$step_id      = filter_input( INPUT_POST, 'step_id', FILTER_VALIDATE_INT );
		$product_id   = filter_input( INPUT_POST, 'product', FILTER_VALIDATE_INT );
		$checker      = filter_input( INPUT_POST, 'checker', FILTER_SANITIZE_STRING );
		$quantity     = filter_input( INPUT_POST, 'quantity', FILTER_SANITIZE_STRING );
		$key          = filter_input( INPUT_POST, 'key', FILTER_SANITIZE_STRING );
		$is_lms       = filter_input( INPUT_POST, 'is_lms', FILTER_SANITIZE_STRING );
		$user_id      = filter_input( INPUT_POST, 'user_id', FILTER_SANITIZE_STRING );
		$funnel_id    = Wpfnl_functions::get_funnel_id_from_step( $step_id );
		$type         = get_post_meta( $funnel_id, '_wpfnl_funnel_type', true );
		$type         = !$type ? 'wc' : $type;
		$class_object = Wpfnl_Public_Type_Factory::build( $type );
		if( $class_object ) {
			$data = $class_object->wpfnl_order_bump_trigger( $step_id, $product_id, $quantity, $key, $user_id, $funnel_id, $checker );
			if( $type === 'lms' ) {
				wp_send_json( $data );
			}
			elseif( $type === 'wc' ) {
				add_filter( 'woocommerce_locate_template', [ new Wpfnl_Public(), 'wpfunnels_woocommerce_locate_template' ], 20, 3 );
				wp_send_json( Wpfnl_functions::get_wc_fragments( $data ) );
			}
		}
	}


	/**
	 * Get replaceable orderbump products
	 */
	private function get_replaceable_ob_products( $ob_settings ){


		$replaceable_ob = [];
		foreach( $ob_settings as $key=>$settings ){
			if( $settings['isReplace'] ===  'yes' ){
				$replaceable_ob[] = [
					'id'       => $settings['product'],
					'quantity' => $settings['quantity'],
				];
			}
		}

		return $replaceable_ob;

	}


	/**
	 * get calculable price
	 *
	 * @param \WC_Product $product
	 * @param $discount_apply
	 * @return string
	 *
	 * @since 2.0.5
	 */
	private function get_product_price( \WC_Product $product, $discount_apply = 'regular' ) {
		$price = $product->get_regular_price();
//		if( is_plugin_active( 'woocommerce-subscriptions/woocommerce-subscriptions.php' ) ){
//			$signUpFee 	= \WC_Subscriptions_Product::get_sign_up_fee( $product );
//			$price 		= $price + $signUpFee;
//		}
		return $discount_apply === 'sale' && $product->get_sale_price() ? $product->get_sale_price() : $price;
	}


	/**
	 * add checkout products to cart
	 *
	 * @param $checkout_id
	 * @throws \Exception
	 */
	private function add_checkout_products($checkout_id)
	{
		$checkout_products = get_post_meta( $checkout_id, '_wpfnl_checkout_products', true );
		foreach ($checkout_products as $product) {
			$_product_id = $product['id'];
			$_qty = $product['quantity'];
			WC()->cart->add_to_cart($_product_id, $_qty);
		}
	}

	/**
	 * add last updated checkout products to cart
	 *
	 * @param $updated_cart
	 * @throws \Exception
	 */
	private function update_checkout_with_last_updated_cart($updated_cart){
		foreach ($updated_cart as $cart) {
			WC()->cart->add_to_cart( $cart['product_id'], $cart['quantity'], $cart['variation_id'], $cart['variation'], $cart['line_total']);
		}
	}


	public function wpfnl_checkout_cart()
	{
		$values = [];
		parse_str(sanitize_text_field($_POST['post_data']), $values);
		$cart = $values['cart'];
		foreach ($cart as $cart_key => $cart_value) {
			WC()->cart->set_quantity($cart_key, $cart_value['qty'], false);
			WC()->cart->calculate_totals();
			woocommerce_cart_totals();
		}
		wp_die();
	}


	/**
	 * Order Bump Templates
	 *
	 * @param $settings
	 * @since 1.0.0
	 */
	public function render_order_bump_template($settings)
	{
		if (!empty($settings['selectedStyle'])) {
			require_once WPFNL_DIR . 'public/modules/checkout/templates-style/order-bump-template-' . $settings['selectedStyle'] . '.php';
		}
	}


	/**
	 * display coupon field or not
	 *
	 * @since 2.0.5
	 */
	public function display_coupon_field() {
		if ( Wpfnl_functions::check_if_this_is_step_type('checkout') ) {
			global $post;
			$checkout_id = $post->ID;
			$show_coupon = Wpfnl::get_instance()->meta->get_checkout_meta_value( $checkout_id, '_wpfnl_checkout_coupon' );
			if( 'no' === $show_coupon ) {
				remove_action( 'woocommerce_before_checkout_form', 'woocommerce_checkout_coupon_form' );
			}
		}
	}

	/**
	 * Out of Stock message in checkout page
	 * @param $message
	 * @param $product_data
	 * @return string
	 */

	public function wpfnl_out_of_stock_message_in_checkout( $message, $product_data ){
		$message = __( 'Sorry, the following product(s) that you are willing to purchase is out of stock at the moment.', 'woocommerce' );
		$image = wp_get_attachment_image_src( get_post_thumbnail_id( $product_data->get_id() ), array( 100, 100) );
		$message .= '<div class="wpfnl-checkout-stock-message"><img src="'.$image[0].'"><h4>'.$product_data->get_title().' </h4></div>';
		return $message;
	}


	/**
	 * Remove Astra theme checkout shipping hook
	 */
	public function remove_astra_theme_checkout_shipping_hook(){
		return true;
	}


	/**
	 * Update quantity from checkout page
	 *
	 * @param $quantity, $cart_item, $cart_item_key
	 */
	public function wpfnl_checkout_cart_item_quantity( $quantity, $cart_item, $cart_item_key ) {

		$step_id = 0;
		$isQuantity = 'no';

		if( wp_doing_ajax() ) {
			$step_id        = isset($_POST['step_id']) ? $_POST['step_id'] : 0;
		} else {
			$step_id = get_the_ID();
		}


		$isQuantity = get_post_meta($step_id, '_wpfnl_quantity_support',true);
		$order_bump_product = get_post_meta($step_id,'order-bump-settings',true);

		if($isQuantity === 'yes'){
			if(isset($order_bump_product['product']) && isset($order_bump_product['isEnabled'])){

				if( ($order_bump_product['product'] == $cart_item["product_id"]) && $order_bump_product['isEnabled'] == 'yes' ){
					return $quantity;
				}
				$variations = json_encode($cart_item['variation']);
				$product_id = $cart_item["product_id"];
				$quantity = $cart_item["quantity"];
				$variation_id = $cart_item["variation_id"];
				$quantity = "× <input type='number' min='1' value='".$quantity."' class='wpfnl-quantity-setect' data-product-id='".$product_id."' data-variation='".$variations."' data-variation-id='".$variation_id."'/>";
			}
		}
		return $quantity;

	}


	public function before_checkout_form_actions( $checkout_id ) {

		if( Wpfnl_functions::is_wc_active() ){

			remove_all_actions( 'woocommerce_checkout_billing' );
			remove_all_actions( 'woocommerce_checkout_shipping' );

			add_action( 'woocommerce_checkout_billing', array( WC()->checkout, 'checkout_form_billing' ) );
			add_action( 'woocommerce_checkout_shipping', array( WC()->checkout, 'checkout_form_shipping' ) );
		}

	}


	/**
	 * Set checkout ID hidden field in checkout page.
	 *
	 * @param array $checkout
	 * @return void
	 */
	public function wpfnl_set_checkout_id( $checkout ) {
		global $post;
		$checkout_id = 0;
		if ( Wpfnl_functions::is_funnel_checkout_page() ) {
			if( isset($post->ID) ){
				$checkout_id = $post->ID;
			}else{
				$checkout_id = isset($_POST['step_id']) ? $_POST['step_id'] : '';
			}

		}
		if( $checkout_id ){
			echo '<input type="hidden" class="input-hidden _wpfunnels_checkout_id" name="_wpfunnels_checkout_id" value="' . intval( $checkout_id ) . '">';
		}
	}


	/**
	 * Redirect funnel checkout after login in checkout page
	 *
	 * @param $redirect_url
	 * @param $user
	 * @return false|string
	 */
	public function wpfnl_redirect_after_login( $redirect_url, $user ) {

		if (isset($_POST['_wpfunnels_checkout_id'])) {
			$funnel_checkout_id = $_POST['_wpfunnels_checkout_id'];
			$redirect_url = get_the_permalink($funnel_checkout_id);
		}
		return $redirect_url;

	}


	/**
     * Get Custom  Woocommerce template
     * @param $template
     * @param $template_name
     * @param $template_path
     * @return mixed|string
     */

    public static function wpfunnels_woocommerce_locate_template($template, $template_name, $template_path)
    {
        global $woocommerce;
        $_template 		= $template;
        $plugin_path 	= WPFNL_DIR . '/woocommerce/templates/';

        if (file_exists($plugin_path . $template_name)) {
            $template = $plugin_path . $template_name;
        }

        if ( ! $template ) {
            $template = $_template;
        }

        return $template;
    }


	/**
	 * Discount on checkout
	 *
	 * @param $cart_object
	 */
	public function wpfnl_checkout_discount( $cart_object ){
		if ( is_admin() && !wp_doing_ajax() ){
			return;
		}
		$step_id = '';

		$step_id = Wpfnl_functions::get_checkout_id_from_post( $_POST );

		if( !$step_id ){
			$step_id = isset($_POST['step_id']) ? $_POST['step_id'] : '';
		}
		if( $step_id ) {
			$funnel_id = Wpfnl_functions::get_funnel_id_from_step( $step_id );
			if( $funnel_id ){
				$discount = Wpfnl_functions::discount_on_checkout( $step_id );
				$main_products = get_post_meta($step_id, '_wpfnl_checkout_products', true );
				$is_gbf = get_post_meta( $funnel_id, 'is_global_funnel', true);
				if( 'yes' === $is_gbf ){
                    if(isset( $_COOKIE['wpfunnels_global_funnel_product'] ) ){
						$main_products  = json_decode( wp_unslash( $_COOKIE['wpfunnels_global_funnel_product'] ), true );
					}
                }

				$total = Wpfnl_functions::calculate_main_product_total( $main_products , $cart_object->cart_contents, $step_id );
				if( is_array($discount) ){
					if( !isset($discount['discountOptions']) || (isset($discount['discountOptions']) &&  $discount['discountOptions'] !== 'original') ){
						$discount_amount = Wpfnl_functions::calculate_discount_amount( $discount['discountOptions'], $discount['mutedDiscountValue'], $total );
						$discount_amount = filter_var($discount_amount, FILTER_SANITIZE_NUMBER_FLOAT, FILTER_FLAG_ALLOW_FRACTION);
						$discount_amount = '-'.$discount_amount;
						$cart_object->add_fee( __('Discount'), $discount_amount, false);

					}
				}
			}
		}
	}

	/**
	 * Exclude tax from fee
	 *
	 * @param Array
	 * @param String
	 * @param Obj
	 *
	 * @return mix
	 */
	public function exclude_cart_fees_taxes($taxes, $fee, $cart)
	{
		if (wc_tax_enabled()) {
			if ( wc_prices_include_tax() ) {
				return [];
			}
		}
		return $taxes;
	}

	/**
	 * Get price type
	 */
	public function get_product_price_type( $funnel_id, $step_id ){

		$is_gbf = get_post_meta( $funnel_id, 'is_global_funnel', true );
		if( $is_gbf == 'no' || !$is_gbf ){
			$discount = Wpfnl_functions::discount_on_checkout( $step_id );

			if( is_array($discount) ){
				if( $discount['discountOptions'] !== 'original' ){
					if( $discount['discountapplyto'] == '' || $discount['discountapplyto'] == 'regular' ){
						return 'regular';
					}else{
						return 'sale';
					}
				}
			}
		}
		return 'original';
	}

	/**
	 * After optin form submition set cookie data
	 * Use cookes data for checkout billings fields
	 * @param $step_id
	 * @param $post_action
	 * @param $action_type
	 * @param $record
	 */

	public function get_optin_data_checkout($step_id, $post_action, $action_type, $record){
		if(isset($record->form_data['data_to_checkout']) && 'yes' == $record->form_data['data_to_checkout']){
			/** Set Cookie Data */
			$cookie_name        = 'wpfunnels_send_data_checkout';
			$cookie             = isset( $_COOKIE[$cookie_name] ) ? json_decode( wp_unslash( $_COOKIE[$cookie_name] ), true ) : array();
			$cookie['after_optin_submit_send_for_checkout']   = ( isset($record->form_data['email']) && $record->form_data['email'] ) ? $record->form_data : false;
			setcookie( $cookie_name, wp_json_encode( $cookie ), time() + 3600 * 6, '/', COOKIE_DOMAIN );
		}
	}

	/**
	 * Get Optin cookes data in checkout fields
	 * @param $fields
	 * @return array
	 */
	public function set_option_data_in_checkout_filed($fields)
	{

		$cookie_name        = 'wpfunnels_send_data_checkout';
		$cookie             = isset( $_COOKIE[$cookie_name] ) ? json_decode( wp_unslash( $_COOKIE[$cookie_name] ), true ) : array();
		if(!empty($cookie)){
			if(isset($cookie['after_optin_submit_send_for_checkout'])){
				$first_name  	= isset($cookie['after_optin_submit_send_for_checkout']['first_name']) ? $cookie['after_optin_submit_send_for_checkout']['first_name'] : '';
				$last_name  	= isset($cookie['after_optin_submit_send_for_checkout']['last_name']) ? $cookie['after_optin_submit_send_for_checkout']['last_name'] : '';
				$phone   		= isset($cookie['after_optin_submit_send_for_checkout']['phone']) ? $cookie['after_optin_submit_send_for_checkout']['phone'] : '';
				$email   		= isset($cookie['after_optin_submit_send_for_checkout']['email']) ? $cookie['after_optin_submit_send_for_checkout']['email'] : '';
				$fields['billing']['billing_first_name']['default'] = $first_name ;
				$fields['billing']['billing_last_name']['default'] 	= $last_name ;
				$fields['billing']['billing_phone']['default'] 		= $phone ;
				$fields['billing']['billing_email']['default'] 		= $email ;
			}
		}
		return $fields;
	}

    /**
     * Hide discount section from checkout for recurring totals
     * @param String $next_pament_date
     * @param Object $recurring_carts
     * 
     * @return Object $recurring_carts
     * @since 2.5.5
     */
    // public function remove_discount_from_recurring_totals( $next_payment_date, $recurring_carts){
    //     $fees = $recurring_carts->get_fees();
    //     $discount = 0;
    //     foreach ($fees as $key => $fee) {
    //         if( isset($fees[$key]->name) && __( 'Discount','wpfnl') === $fees[$key]->name) {
    //             // $discount = $fees[$key]->value;
    //             unset($fees[$key]);
    //         }
    //     }
    //     $recurring_carts->fees_api()->set_fees($fees);
    //     return $recurring_carts;
    // }

}

